const chokidar = require('chokidar')
const fs = require('fs-extra')
const pathComm = require('path')
const chalk = require('chalk')
const modify = require('./modify')
const { materielData } = require('./data')

/**
 * 遗留问题：
 * 1、同名组件导致babel编译失败
 * 2、修改单个文件导致全量更新
 * 3、组件不规范校验
 * 4、启动后触发多次copy
 * 5、第三方组件修改后应重新刷新页面
 */
const resolve = (dir) => pathComm.join(__dirname, '../', dir)

const inData = (pathArr, path) => {
    return pathArr.map((v) => {
        const comPath = `${path}/${v}`
        const d = {
            designPath: pathComm.joinj(comPath, '/design'),
            view: pathComm.joinj(comPath, '/view'),
            jsonPath: pathComm.joinj(comPath, '/index.json')
        }
        const json = JSON.parse(fs.readFileSync(d.jsonPath))
        return { ...d, fieldType: json.data.fieldType }
    })
}
const componentsPath = pathComm.join(process.cwd(), '/components')
const materielPath = resolve('/src/materiel')

const findJson = (toSrc) => {
    return new Promise((res) => {
        const paths = fs.readdirSync(toSrc)
        const jsonPaths = []
        const comName = []
        const pathNames = []
        paths.forEach(async (path) => {
            const materielAllSrc = pathComm.join(materielPath, `/${path}`)
            const status = await fs.statSync(materielAllSrc)
            if (status.isDirectory()) {
                const depPaths = fs.readdirSync(materielAllSrc)
                depPaths.forEach((depPath, i) => {
                    if (depPath === 'index.json') {
                        const jsonPath = pathComm.join(materielAllSrc, `/${depPath}`)
                        jsonPaths.push(jsonPath)
                        pathNames.push(path)
                        const json = JSON.parse(fs.readFileSync(jsonPath))
                        comName.push(json.data.fieldType)
                    }
                    if (i === depPaths.length - 1) {
                        res({ jsonPaths, comName, pathNames })
                    }
                })
            }
        })
    })
}

const filesUtils = {
    modifyFunc: async (toSrc) => {
        const { jsonPaths, comName, pathNames } = await findJson(toSrc)
        materielData['comKey'] = {
            jsonPaths,
            comName,
            pathNames
        }
        modify(resolve('src/materiel/view.ts'), comName, pathNames, 'view')
        modify(resolve('src/materiel/design.ts'), comName, pathNames, 'design')
        modify(resolve('src/materiel/data.ts'), comName, pathNames, 'index.json')
        console.log(chalk.green('materiel 初始化完成'))
    },
    copy: (targetSrc, toSrc) => {
        console.log(chalk.green(`发现组件: ${targetSrc} ---> ${toSrc}\n`))
        fs.copySync(targetSrc, toSrc)
        filesUtils.modifyFunc(toSrc)
    },
    addDir: (targetSrc, toSrc) => {
        filesUtils.copy(targetSrc, toSrc)
    },
    unlinkDir: (targetSrc, toSrc, path) => {
        const arr = path.split(componentsPath)
        fs.removeSync(pathComm.join(materielPath, arr[1]))
        filesUtils.modifyFunc(toSrc)
    },
    add: (targetSrc, toSrc) => {
        filesUtils.copy(targetSrc, toSrc)
    },
    change: (targetSrc, toSrc) => {
        filesUtils.copy(targetSrc, toSrc)
    },
    unlink: (targetSrc, toSrc, path) => {
        const arr = path.split(componentsPath)
        fs.removeSync(pathComm.join(materielPath, arr[1]))
        filesUtils.modifyFunc(toSrc)
    }
}

module.exports = () => {
    chokidar.watch(`${process.cwd()}/components`, {}).on('all', async (event, path) => {
        filesUtils[event](componentsPath, materielPath, path)
    })

    chokidar
        .watch(process.cwd(), {
            ignored: /node_modules|.eslintrc.js|.prettierrc|.stylelintrc|package|package-lock.json|components/
        })
        .on('all', (event, path) => {
            console.log('all', event, path)
        })
}
