const fs = require('fs-extra')
const pathComm = require('path')
const chalk = require('chalk')
const babel = require('@babel/parser')
const traverse = require('@babel/traverse').default
const generate = require('@babel/generator').default
const prettier = require('prettier')
const t = require('@babel/types')
const codes = require('./default-code')

const { parse } = babel

const resolve = (dir) => pathComm.join(__dirname, '../', dir)

const parseJsx = (code) => {
    return parse(code, { sourceType: 'module', plugins: ['jsx', 'typescript'] })
}

const codeStr = codes.formCodeStr

function unique(arr) {
    for (var i = 0; i < arr.length; i++) {
        for (var j = i + 1; j < arr.length; j++) {
            if (arr[i].fieldType == arr[j].fieldType) {
                arr.splice(j, 1)
                j--
            }
        }
    }
    return arr
}

const tFunc = {
    String: t.stringLiteral,
    Number: t.numericLiteral,
    Boolean: t.booleanLiteral,
    Array: (arr) => {
        let arEps = []
        arr.forEach((v) => {
            arEps.push(tFunc[v.constructor.name](v))
        })
        return t.arrayExpression(arEps)
    },
    Object: (data) => t.objectExpression(generateObj(data))
}

const generateObj = (data) => {
    let ops = []
    Object.keys(data).forEach((v) => {
        if (data[v] === undefined || data[v] === null) {
            ops.push(t.objectProperty(t.identifier(v), t.stringLiteral(`${data[v]}`)))
            return
        }
        const typeName = data[v].constructor.name
        if (!tFunc[typeName]) {
            console.log(chalk.red(`请查看组件<${data.fieldType}> index.json 存在不支持的数据类型${typeName}`))
        }
        ops.push(t.objectProperty(t.identifier(v), tFunc[typeName](data[v])))
    })
    return ops
}

const tfCode = (code, list, formLayout) => {
    const ast = parseJsx(code)
    let codeImport
    let rootNode
    let varPath
    traverse(
        ast,
        {
            Program(path) {
                ProgramPath = path
            },
            VariableDeclaration(path) {
                console.log(path.node.declarations[0].id.name)
                if (path.node.declarations[0].id.name === 'data') {
                    varPath = path
                }
            },
            JSXOpeningElement(path) {
                if (t.isJSXIdentifier(path.node.name, {name: 'Row'})) {
                    rootNode = path.parent
                }
                if (t.isJSXIdentifier(path.node.name, {name: 'Form'})) {
                    path.node.attributes.push(t.jsxSpreadAttribute(t.identifier('formLayout')))
                }
            },
            JSXElement(path) {
                // rootNode = path.node
            },
            ImportDeclaration(path) {
                codeImport = path
            }
        },
        null
    )

    varPath.insertAfter(
        t.variableDeclaration('var', [
            t.variableDeclarator(t.identifier('formLayout'), t.objectExpression(generateObj(formLayout)))
        ])
    )

    const pathData = unique(
        list.map((v) => {
            return { fieldType: v.fieldType, viewPath: v.viewPath }
        })
    )

    pathData.forEach((v) => {
        const imTion = t.importDeclaration(
            [t.importDefaultSpecifier(t.identifier(v.fieldType))],
            t.stringLiteral(`@${v.viewPath}`)
        )
        codeImport.insertAfter(imTion)
    })

    list.forEach((v) => {
        console.log(chalk.green(`发现${v.fieldType}`))
        const dataName = `Data${v.fieldKey}`
        const objEp = t.objectExpression(generateObj(v))
        varPath.insertAfter(
            t.variableDeclaration('var', [t.variableDeclarator(t.identifier(dataName), objEp)])
        )

        const colJsx = t.jsxElement(
            t.jsxOpeningElement(t.jsxIdentifier('Col'), [
                t.jsxSpreadAttribute(t.memberExpression(t.identifier(dataName), t.identifier('colProps')))
            ]),
            t.JSXClosingElement(t.JSXIdentifier('Col')),
            [t.jsxText('')],
            true
        )

        const jEp = t.jsxExpressionContainer(t.identifier(dataName))
        const joe = t.jsxOpeningElement(t.jsxIdentifier(v.fieldType), [
            t.jsxAttribute(t.jsxIdentifier('data'), jEp),
            t.jsxAttribute(t.jsxIdentifier('form'), t.jsxExpressionContainer(t.identifier('form'))),
            t.jsxSpreadAttribute(t.identifier('props'))
        ])

        const jsx = t.jsxElement(
            joe,
            t.JSXClosingElement(t.JSXIdentifier(v.fieldType)),
            [t.jsxText('')],
            true
        )
        colJsx.children.push(jsx)
        rootNode.children.push(colJsx)
    })
    varPath.remove()
    return generate(ast)
}

module.exports = (formName) => {
    return new Promise((res, rej) => {
        try {
            const tsxPath = resolve('src/component/generate/index.tsx')
            const metaPath = pathComm.join(resolve('/cli/meta-file'), `${formName}.json`)
            const { list = [], formLayout } = JSON.parse(fs.readFileSync(metaPath))
            if (!list.length) {
                console.log(chalk.red('打包失败,请保存数据\n'))
                throw Error('打包失败,请保存数据')
                // return
            }
            console.log(chalk.green('正在生成组件... \n'))
            fs.writeFileSync(tsxPath, codeStr)
            const { code } = tfCode(fs.readFileSync(tsxPath, 'utf8'), list, formLayout)
            fs.writeFileSync(tsxPath, prettier.format(code, { parser: 'babylon' }))
            console.log(chalk.green('打包完成...\n'))
            res(code)
        } catch (err) {
            console.error(err)
            rej(err)
        }
    })
}
