import React, { useRef } from 'react'

import { useDrop, useDrag, DragSourceMonitor } from 'react-dnd'

import Icon from '@/component/edit-engine/component/iconfont/icon'

import isEqual from 'lodash.isequal'

import './index.less'

const DndItem = ({ item, index, beginDrag, endDrag, dragItem, children, keyName }) => {
    const ref = useRef<HTMLDivElement>(null)

    const [{ isDragging }, drag, dragPreview] = useDrag({
        collect: (monitor: DragSourceMonitor) => ({
            isDragging: monitor.isDragging()
        }),
        item: {
            type: 'Design',
            id: item[keyName],
            index
        },
        begin: (props) => {
            beginDrag(item[keyName])
        },
        end: (props, monitor) => {
            endDrag(item[keyName])
        }
    })

    const [, drop] = useDrop({
        accept: 'Design',
        hover: (props, monitor) => {
            const { id: draggedId } = monitor.getItem()
            const overId = item[keyName]

            if (draggedId && overId && draggedId !== overId) {
                dragItem(draggedId, overId)
                return
            }
        }
    })

    dragPreview(drop(ref))
    return drag(
        <div className="drag-btn" ref={ref}>
            {/* <div className="move-btn">
                <Icon name="icon-Icon_tuodong_px"></Icon>
            </div> */}
            {children}
        </div>
    )
}

const Design = ({
    className,
    variable,
    list,
    dragItem,
    renderDom,
    keyName = '',
    beginDrag = () => {},
    endDrag = () => {}
}) => {
    const [, drop] = useDrop({
        accept: 'Design'
    })
    return (
        <div className={`cb-que-sys_drag ${className}`} ref={drop}>
            {list.map((v, i) => {
                const keyN = keyName || 'fieldKey'
                const key = v[keyN]
                return (
                    <DndItem
                        key={key}
                        item={v}
                        dragItem={dragItem}
                        keyName={keyN}
                        beginDrag={beginDrag}
                        endDrag={endDrag}
                        index={i}
                    >
                        {renderDom(v, i, variable)}
                    </DndItem>
                )
            })}
        </div>
    )
}

export const areEqual = (prev, next) => {
    if (!((prev && prev.list && prev.list.length) || (next && next.list && prev.list.length))) {
        return false
    }
    const equal = isEqual(prev.list, next.list)
    if (equal) {
        return true
    }
    return false
}

export default React.memo(Design, areEqual)
