import React  from 'react'

import isEqual from 'lodash.isequal'
import { throttle } from '@/component/edit-engine/utils'
import { useDrop } from 'react-dnd'
import { Form, Row, Col } from 'antd'

import Item from './item'

import Doms from '@/materiel/view'

interface Iprops extends cbQueySys.mainProps {
    checkedKey: string
    beginDrag?: (key: string) => void
    endDrag?: (key: string) => void
    sideDragItem?: (type: any, key: any) => void
    disablePage: boolean
    mainHover: (type: string) => void
    formLayout: any
}

const RenderDom = React.memo(
    (props: any) => {
        const { item, i, disablePage, form } = props
        const propsItem = {
            data: item,
            key: item.fieldKey,
            index: i,
            disablePage: disablePage,
            form
        }
        const ItemDom = Doms[item.fieldType]
        if (!ItemDom) {
            return <div></div>
        }
        return <ItemDom key={item.fieldKey} {...propsItem} />
    },
    (prev, next) => {
        if (!((prev && prev.item) || (next && next.item))) {
            return false
        }
        if (prev.item.fieldType === 'NextPage' && next.item.fieldType === 'NextPage') {
            return false
        }
        const equal = isEqual(prev.item, next.item)
        if (equal) {
            return true
        }
        return false
    }
)

export default (props: Iprops) => {
    const [, drop] = useDrop({
        accept: 'main',
        hover: (_, monitor) => {
            const { dragType } = monitor.getItem()
            props.mainHover && props.mainHover(dragType)
        }
    })

    const [form] = Form.useForm()
    return (
        <React.Fragment>
            <div className="cb-que-sys_drag cb-que-sys_main" ref={drop}>
                {props.list.length ? '' : <div className="main-null">从左侧选择或拖动组件进行表单设计</div>}
                <Form form={form} {...props.formLayout}>
                    <Row>
                        {props.list.map((item, index) => (
                            <Col {...item.colProps} key={item.fieldKey}>
                                <Item
                                    key={item.fieldKey}
                                    optChange={props.optChange}
                                    checkedKey={props.checkedKey}
                                    checkedItem={props.checkedItem}
                                    disablePage={props.disablePage}
                                    dragItem={throttle(props.dragItem, 500)}
                                    sideDragItem={throttle(props.sideDragItem, 500)}
                                    beginDrag={props.beginDrag}
                                    endDrag={props.endDrag}
                                    countLength={props.list.length}
                                    index={index}
                                    item={item}
                                >
                                    <RenderDom
                                        form={form}
                                        item={item}
                                        i={index}
                                        disablePage={props.disablePage}
                                        list={props.list}
                                    />
                                </Item>
                            </Col>
                        ))}
                    </Row>
                </Form>
            </div>
        </React.Fragment>
    )
}
