import React, { useRef } from 'react'

import ItemHeader from './item-header'

import { DragSourceMonitor, useDrag, useDrop } from 'react-dnd'

const Item = ({
    children,
    index,
    checkedKey,
    item,
    checkedItem,
    optChange,
    sideDragItem,
    dragItem,
    beginDrag,
    endDrag,
    disablePage,
    countLength
}: any) => {
    const ref = useRef<HTMLDivElement>(null)

    const [{ isDragging }, drag, dragPreview] = useDrag({
        collect: (monitor: DragSourceMonitor) => ({
            isDragging: monitor.isDragging()
        }),
        item: {
            type: 'main',
            id: item.fieldKey,
            dragType: 'mainItem',
            index
        },
        begin: (props) => {
            beginDrag(item.fieldKey)
        },
        end: (props, monitor) => {
            endDrag(item.fieldKey)
        }
    })

    const [, drop] = useDrop({
        accept: 'main',
        hover: (props, monitor) => {
            const { id: draggedId, dragType, index: dragIndex } = monitor.getItem()
            const hoverIndex = index
            const { fieldKey: overId, tipCommon } = item
            if (tipCommon) {
                return
            }

            const hoverBoundingRect = ref.current!.getBoundingClientRect()

            const hoverMiddleY = (hoverBoundingRect.bottom - hoverBoundingRect.top) / 2

            const clientOffset = monitor.getClientOffset()

            const hoverClientY = clientOffset.y - hoverBoundingRect.top

            /** 向上和向下拖动。光标超过50%才移动。防止鼠标位于元素一半高度时元素抖动的状况 */
            // 向下拖动
            if (dragIndex < hoverIndex && hoverClientY < hoverMiddleY) {
                return
            }

            // 向上拖动
            if (dragIndex > hoverIndex && hoverClientY > hoverMiddleY) {
                return
            }
            if (draggedId && overId && draggedId !== overId && dragType === 'mainItem') {
                dragItem(draggedId, overId)
                return
            }
            if (dragType !== 'mainItem') {
                sideDragItem && sideDragItem(dragType, overId)
            }
        }
    })

    dragPreview(drop(ref))

    return (
        <div
            ref={ref}
            className={`sys_drag_item ${checkedKey === item.fieldKey && 'checked-item '}`}
            onClick={() => {
                checkedItem && checkedItem(item)
            }}
            id={`drag-${item.fieldKey}`}
        >
            {drag(
                <div className={`drag-item_box ${item.fieldKey === checkedKey && 'checked-drag_item'}`}>
                    <div className="move-msk"></div>
                    {children}
                    <div className="item-tools">
                        <ItemHeader
                            data={item}
                            optChange={optChange}
                            disablePage={disablePage}
                            countLength={countLength}
                            index={index}
                        />
                    </div>
                    {item.tipCommon ? <div className="tip-common"></div> : ''}
                </div>
            )}
        </div>
    )
}

export default Item
