import React, { useEffect, useImperativeHandle, forwardRef, useCallback } from 'react'

import { useImmer } from 'use-immer'

import Side from './commonts/side'
import Main from './commonts/main'
import Design from './commonts/design'
import DragAndDropHooc from '@/component/edit-engine/component/drag-common/drag-and-drop-hooc'
import isEqual from 'lodash.isequal'

import { createEditFormItem, getDragIndex, notChangeDesc } from '../../utils/index'

import data from '@/component/edit-engine/data'

import './index.less'

import designData from '@/materiel/data'

interface stateTypes {
    formFieldList: cbQueySys.mainItemType[]
    ckItem: cbQueySys.mainItemType
    headerData: any
    addItem: {
        index: number
        data: cbQueySys.mainItemType
    }
}

const useStateFunc = (props: any) => {
    const [state, setState] = useImmer<stateTypes>({
        formFieldList: [],
        ckItem: {} as any,
        addItem: {
            index: undefined,
            data: {} as any
        },
        headerData: {}
    })

    const sideItemClick = (item: cbQueySys.sildemodelsItem) => {
        setState((draft) => {
            /** 分页标题不需要计算到题目中，这里过滤一下 */
            const len = draft.formFieldList.reduce((p, v) => {
                if (notChangeDesc[v.fieldType]) {
                    return p
                }
                return p + 1
            }, 1)
            const data = createEditFormItem(designData[item.key].data, len)
            const ckData = JSON.parse(JSON.stringify(data))
            draft.formFieldList.push(ckData)
            draft.ckItem = ckData
        })
    }

    const checkedItem = (item: cbQueySys.mainItemType) => {
        setState((draft) => {
            draft.ckItem = item
        })
    }

    const btnMoveItem = (index: number, type) => {
        const goIndex = type === 'up' ? index - 1 : index + 1
        setState((draft) => {
            draft.formFieldList.splice(goIndex, 0, draft.formFieldList.splice(index, 1)[0])
        })
    }

    const removeItem = (index: number) => {
        setState((draft) => {
            draft.formFieldList.splice(index, 1)
            draft.ckItem = draft.formFieldList.length
                ? draft.formFieldList[draft.formFieldList.length - 1]
                : ({} as any)
        })
    }

    const optChange = (type: cbQueySys.optType, item: cbQueySys.mainItemType) => {
        const index = state.formFieldList.findIndex((v) => v.fieldKey === item.fieldKey)
        const func = {
            remove: removeItem,
            up: btnMoveItem,
            down: btnMoveItem
        }
        func[type](index, type)
        props.change && props.change(true)
        postData()
    }

    const dragMoveItem = (targetKey: string, goKey: string) => {
        const i = getDragIndex(state.formFieldList, targetKey, goKey)
        setState((draft) => {
            const v = draft.formFieldList.splice(i.targetIndex, 1)
            draft.formFieldList.splice(i.goIndex, 0, v[0])
        })
        props.change && props.change(true)
        postData()
    }

    const sideDragMoveItem = (type: string, key: string) => {
        const index = state.formFieldList.filter((v) => !v.tipCommon).findIndex((v) => v.fieldKey === key)
        const len = state.formFieldList.reduce((p, v) => {
            if (notChangeDesc[v.fieldType]) {
                return p
            }
            return p + 1
        }, 1)
        const data = createEditFormItem(designData[type].data, len)
        const ckData = JSON.parse(JSON.stringify(data))
        setState((draft) => {
            if (draft.addItem.index !== undefined && draft.addItem.index === index) {
                /** 避免多次触发 */
                return
            }
            /** 每次删除上次的 */
            draft.formFieldList = draft.formFieldList.filter((v) => !v.tipCommon)
            draft.formFieldList.splice(index, 0, { ...ckData, tipCommon: true })
            draft.addItem = {
                index: index,
                data: ckData
            }
        })
    }

    const editChange = (cb, fieldKey: string) => {
        window.requestAnimationFrame(() => {
            setState((draft) => {
                const index = draft.formFieldList.findIndex((v) => v.fieldKey === fieldKey)
                cb(draft.formFieldList[index])
                draft.ckItem = draft.formFieldList[index]
                props.change && props.change(true)
                postData()
            })
        })
        return
    }

    useEffect(() => {
        const { data = { headerData: {} } } = props
        if (!(data && data.headerData && data.headerData.formName)) {
            return
        }
        setState((draft) => {
            draft.formFieldList = data.formFieldList || []
            draft.headerData = data.headerData
        })
    }, [props.data])

    const getSuccessData = () => {
        return JSON.parse(
            JSON.stringify({
                ...state.headerData,
                formFieldList: state.formFieldList
            })
        )
    }

    const beginDrag = (key: string) => {
        if (document.querySelector('.drag-dom')) {
            return
        }
        const dragDom = document.querySelector(`#drag-${key}`)
        dragDom.classList.add('drag-dom')
    }

    const endDrag = (key: string) => {
        const dragDom = document.querySelector(`#drag-${key}`)
        dragDom.classList.remove('drag-dom')
    }

    const postData = () => {}

    const sideEndDrag = (didDrop: boolean) => {
        if (didDrop) {
            setState((draft) => {
                draft.formFieldList = draft.formFieldList.filter((v) => !v.tipCommon)
                draft.formFieldList.splice(draft.addItem.index, 0, draft.addItem.data)
                draft.addItem = {
                    index: undefined,
                    data: {} as any
                }
            })
            props.change && props.change(true)
            postData()
            return
        }
        setState((draft) => {
            // const index = draft.formFieldList.filter((v) => !v.tipCommon)
            draft.formFieldList = draft.formFieldList.filter((v) => !v.tipCommon)
            draft.addItem = {
                index: undefined,
                data: {} as any
            }
        })
        postData()
    }

    const mainHover = (type: string) => {
        if (state.formFieldList.length > 2) {
            return
        }
        const ckData = JSON.parse(JSON.stringify(createEditFormItem(designData[type].data, 1)))
        setState((draft) => {
            if (draft.addItem.index !== undefined) {
                /** 避免多次触发 */
                return
            }
            draft.addItem = {
                index: 0,
                data: ckData
            }
            draft.formFieldList.push({
                ...ckData,
                tipCommon: true
            })
        })
    }

    const sideBeginDrag = (didDrop) => {}

    const getSideList = (): cbQueySys.sildeList[] => {
        if (designData) {
            const list: cbQueySys.sildeList[] = [
                {
                    name: '自定义组件',
                    models: []
                }
            ]
            Object.values(designData).forEach((v) => {
                list[0].models.push({
                    name: v.data.fieldTypeName,
                    key: v.data.fieldType
                })
            })
            return list
        }
        return []
    }

    return {
        state,
        sideItemClick,
        checkedItem,
        dragMoveItem,
        sideDragMoveItem,
        editChange,
        getSuccessData,
        optChange,
        beginDrag,
        endDrag,
        sideEndDrag,
        sideBeginDrag,
        getSideList,
        mainHover
    }
}

const CbQueSys = (props: cbQueySys.Iprops, ref: any) => {
    const {
        state,
        sideItemClick,
        checkedItem,
        dragMoveItem,
        sideDragMoveItem,
        editChange,
        getSuccessData,
        optChange,
        beginDrag,
        endDrag,
        sideEndDrag,
        sideBeginDrag,
        getSideList,
        mainHover
    } = useStateFunc(props)

    useImperativeHandle(ref, () => ({
        getSuccessData
    }))

    return (
        <div className="cb-que-sys">
            <DragAndDropHooc>
                <div className="cb-que-sys_middle">
                    <div className="sys-left">
                        <Side
                            sideBeginDrag={sideBeginDrag}
                            sideEndDrag={sideEndDrag}
                            sideItemClick={sideItemClick}
                            list={getSideList()}
                            disablePage={props.disablePage}
                        />
                    </div>
                    <div className="sys-main">
                        <Main
                            list={state.formFieldList}
                            formLayout={props.formLayout}
                            checkedItem={checkedItem}
                            checkedKey={state.ckItem.fieldKey}
                            optChange={optChange}
                            dragItem={dragMoveItem}
                            mainHover={mainHover}
                            sideDragItem={sideDragMoveItem}
                            beginDrag={beginDrag}
                            endDrag={endDrag}
                            disablePage={props.disablePage}
                        />
                    </div>
                    <div className="sys-right">
                        <Design data={state.ckItem} onChange={editChange} disablePage={props.disablePage} />
                    </div>
                </div>
            </DragAndDropHooc>
        </div>
    )
}

const CbQueSysCommon = forwardRef(CbQueSys)

export default CbQueSysCommon
