export const typeNames = {
    Checkbox: '多选',
    CitySelect: '城市选择',
    Date: '日期选择',
    Input: '输入框',
    Title: '标题',
    TitleReset: '标题',
    NextPage: '分页',
    MatrixRadio: '矩阵单选',
    MatrixCheckbox: '矩阵多选',
    TextArea: '输入框多行',
    Time: '时间选择',
    Upload: '图片上传',
    Select: '下拉框',
    Radio: '单选'
}

const keys: any = {}

export const createUid = (): string => {
    const key = Number(
        Math.random()
            .toString()
            .substr(3, length) + Date.now()
    ).toString(36)
    if (!keys[key]) {
        keys[key] = key
        return key
    }
    return createUid()
}

const isArray = (item: any): boolean => {
    return Object.prototype.toString.call(item) === '[object Array]'
}

const setItemKey = (item: any[]): any[] => {
    if (!isArray(item)) {
        return item
    }
    return item.map((v) => {
        v.fieldKey = createUid()
        if (v && v.fieldOption) {
            v.fieldOption = v.fieldOption.map((j) => {
                j.fieldKey = createUid()
                return j
            })
        }
        return v
    })
}

export const notChangeDesc = {
    Title: true,
    TitleReset: true,
    NextPage: true
}

export const createEditFormItem = (item: any, len: number): any => {
    return {
        ...item,
        formParams: { ...(item.formParams || {}), field: createUid() },
        fieldDesc: notChangeDesc[item.fieldType] ? item.fieldDesc : `${item.fieldDesc}${len}`,
        fieldKey: createUid(),
        fieldOption: setItemKey(item.fieldOption),
        children: setItemKey(item.children)
    }
}

// index是当前元素下标，tindex是目标。
export const moveArr = (arr, index, tindex) => {
    arr.split()
    if (index > tindex) {
        arr.splice(tindex, 0, arr[index])
        arr.splice(index + 1, 1)
    } else {
        arr.splice(tindex + 1, 0, arr[index])
        arr.splice(index, 1)
    }
}

export function throttle(fn, wait) {
    var pre = Date.now()
    return function() {
        var context = this
        var args = arguments
        var now = Date.now()
        if (now - pre >= wait) {
            fn.apply(context, args)
            pre = Date.now()
        }
    }
}

// 获取当前是pc还是移动端
export function IsPC() {
    var userAgentInfo = navigator.userAgent
    var Agents = ['Android', 'iPhone', 'SymbianOS', 'Windows Phone', 'iPad', 'iPod']
    var flag = true
    for (var v = 0; v < Agents.length; v++) {
        if (userAgentInfo.indexOf(Agents[v]) > 0) {
            flag = false
            break
        }
    }
    return flag
}

export const getDragIndex = (
    list = [] as any[],
    targetKey = '',
    goKey = '',
    keyName = 'fieldKey'
): { targetIndex: number; goIndex: number } => {
    let targetIndex = 0
    let goIndex = 0
    list.forEach((v, i) => {
        if (v[keyName] === targetKey) {
            targetIndex = i
        }
        if (v[keyName] === goKey) {
            goIndex = i
        }
    })
    return { targetIndex, goIndex }
}
