import React from 'react'

import { Input, Select } from 'antd'
import { DesignItem, Icon, throttle } from '@/component/edit-engine/index'

const { Option } = Select

const AyncList = (props) => {
    const { onChangeKeyArr, deleteItem, addItem, dragItem } = props
    const {data} = props
    const dataTypeOpt = () => {
        if (data.designParams.optionsSourceType === 'static') {
            return (
                <>
                    <DesignItem
                        className="edit-item_drag"
                        list={data.fieldOption}
                        variable={{ isCalculate: data.isCalculate }}
                        dragItem={throttle(dragItem, 500)}
                        renderDom={(v, i, variable) => {
                            return (
                                <div className="edit-item_edit">
                                    <div className="item-edit_box">
                                        <div className="item">
                                            <div className="item-label">显示值</div>
                                            <Input
                                                defaultValue={v.label}
                                                placeholder="请输入"
                                                disabled={props.disablePage}
                                                allowClear
                                                onChange={(e) => {
                                                    onChangeKeyArr(
                                                        ['fieldOption', i, 'label'],
                                                        e.target.value
                                                    )
                                                }}
                                            />
                                        </div>
                                        <div className="item">
                                            <div className="item-label">真实值</div>
                                            <Input
                                                defaultValue={v.value}
                                                placeholder="请输入"
                                                disabled={props.disablePage}
                                                allowClear
                                                onChange={(e) => {
                                                    onChangeKeyArr(
                                                        ['fieldOption', i, 'value'],
                                                        e.target.value
                                                    )
                                                }}
                                            />
                                        </div>
                                    </div>
                                    <div
                                        className="item-input_btn"
                                        onClick={() => {
                                            if (props.disablePage) {
                                                return
                                            }
                                            deleteItem(i)
                                        }}
                                    >
                                        <Icon name="icon-_shanchu" />
                                    </div>
                                </div>
                            )
                        }}
                    />
                    <div
                        className="edit-item_add"
                        onClick={() => {
                            if (props.disablePage) {
                                return
                            }
                            addItem()
                        }}
                    >
                        <Icon name="icon-tianjia" color="#fff" />
                        <div className="icon-add_text">添加选项</div>
                    </div>
                </>
            )
        } else if (data.designParams.optionsSourceType === 'dynamic') {
            return (
                <div>
                    <div style={{ margin: '10px 0' }}>数据名称</div>
                    <Input
                        onChange={(e) => {
                            onChangeKeyArr(['designParams', 'optionsSourceName'], e)
                        }}
                        defaultValue={data.designParams.optionsSourceName}
                        placeholder="根据这个名称传入对应数据"
                    ></Input>
                </div>
            )
        } else {
            return (
                <div className="custom-item_box">
                    <div style={{ margin: '10px 0' }}>远程options</div>
                    <div className="custom-item">
                        <div className="label">请求地址</div>
                        <Input defaultValue={data.optionsConfig.url} placeholder="/api" onChange={e => {
                            onChangeKeyArr(['optionsConfig', 'url'], e.target.value)
                        }}></Input>
                    </div>
                    <div className="custom-item">
                        <div className="label">请求方式</div>
                        <Select style={{ width: '100%' }} defaultValue={data.optionsConfig.method} onChange={e => {
                            onChangeKeyArr(['optionsConfig', 'method'], e)
                        }}>
                            <Option value="get">get</Option>
                            <Option value="post">post</Option>
                        </Select>
                    </div>
                    <div className="custom-item">
                        <div className="label">搜索字段名称</div>
                        <Input
                            onChange={e => {
                                onChangeKeyArr(['optionsConfig', 'field'], e.target.value)
                            }}
                            placeholder="接口接收搜索的参数名称"
                            defaultValue={data.optionsConfig.field}
                        ></Input>
                    </div>
                    <div className="custom-item">
                        <div className="label">追加参数</div>
                        <Input.TextArea
                            placeholder="除搜索字段外别的参数"
                            onChange={e => {
                                onChangeKeyArr(['optionsConfig', 'params'], e.target.value)
                            }}
                            defaultValue={data.optionsConfig.params}
                        ></Input.TextArea>
                    </div>
                    <div className="custom-item">
                        <div className="label">返回字段的名称</div>
                        <Input
                            placeholder="返回字段的名称"
                            onChange={e => {
                                onChangeKeyArr(['optionsConfig', 'listName'], e.target.value)
                            }}
                            defaultValue={data.optionsConfig.listName}
                        ></Input>
                    </div>
                </div>
            )
        }
    }
    return (
        <>
            <div className="edit-box_item edit-box_group">
                <div>
                    <div style={{ marginBottom: '10px' }}>数据类型</div>
                    <Select
                        defaultValue={data.designParams.optionsSourceType}
                        onChange={(e) => {
                            onChangeKeyArr(['designParams', 'optionsSourceType'], e)
                        }}
                    >
                        <Option value="static">静态数据</Option>
                        <Option value="dynamic">动态数据</Option>
                        <Option value="custom">自定义数据</Option>
                    </Select>
                </div>
                {dataTypeOpt()}
            </div>
        </>
    )
}

export default AyncList
