import React, { useState, useEffect, useImperativeHandle, forwardRef } from 'react'
import { Form, Row, Col } from 'antd'

import isEqual from 'lodash.isequal'
import Commons from '@/materiel/view'

type Iprops = {
    list: any[]
    formLayout: {
        labelCol: any
        wrapperCol: any
        layout: string
    }
    request: any
    submit: Function
    change: (data: any, val: any) => void
}

const RenderDom = React.memo(
    ({ data, request, change }: { data: any; request: Function; change: Function }) => {
        const Dom = Commons[data.fieldType]
        if (!Dom) {
            return <div className="null-com">空组件</div>
        }
        return <Dom data={data} request={request} change={change} />
    },
    (pre, next) => {
        const equal = isEqual(pre.data, next.data)
        return equal
    }
)

const ViewComm = forwardRef((props: Iprops, ref) => {
    const [newList, setNewList] = useState([])
    const [showIndex, setShowIndex] = useState(0)

    const { list, formLayout, request, change, submit } = props
    const [form] = Form.useForm()

    const nextPageChange = (data, val) => {
        if (val === 'next') {
            setShowIndex(showIndex + 1)
            window.scrollTo(0, 0)
        }
        if (val === 'prev' && showIndex !== 0) {
            setShowIndex(showIndex - 1)
            window.scrollTo(0, 0)
        }
        if (val === 'submit') {
            submit()
        }
    }

    useImperativeHandle(ref, () => ({
        form
    }))

    useEffect(() => {
        if (!list.length) {
            console.error('数据为空')
            alert('空表单,请点保存')
            return
        }
        let index = 0
        const newList = list.map((v) => {
            const item = { ...v, index }
            if (v.fieldType === 'NextPage') {
                index += 1
            }
            return item
        })
        setNewList(newList)
    }, [props.list])

    return (
        <div className="view-comm">
            <Form form={form} {...formLayout}>
                <Row>
                    {newList.map((data) => {
                        return (
                            <Col
                                {...data.colProps}
                                style={{ display: data.index === showIndex ? 'block' : 'none' }}
                            >
                                {data.fieldType !== 'NextPage' ? (
                                    <RenderDom data={data} request={request} change={change} />
                                ) : (
                                    <Commons.NextPage data={data} change={nextPageChange}></Commons.NextPage>
                                )}
                            </Col>
                        )
                    })}
                </Row>
            </Form>
        </div>
    )
})

export default ViewComm
