import React, { useMemo, useState, useEffect } from 'react'

import { Form, Radio } from 'antd'

const formRadio = (props) => {
    const { data } = props
    const [listData, setListData] = useState([])

    const optSource = useMemo(() => {
        const { optionsSourceType, optionsSourceName } = data.designParams || {}
        if (optionsSourceType === 'static') {
            return data.fieldOption || []
        }
        const source = props[optionsSourceName] || []
        if (source && source.length) {
            return props[optionsSourceName]
        }
        console.error(`请传入${optionsSourceName}`)
        return []
    }, [props[data.designParams.optionsSourceName], data.fieldOption])

    const getListData = async (pm = {}) => {
        if (!props.request) {
            return
        }
        const { optionsConfig } = data
        const params = { [optionsConfig.field]: '', ...JSON.parse(optionsConfig.params || '{}'), ...pm }
        const d = optionsConfig.method === 'get' ? { params } : { data: params }
        const res = await props.request(optionsConfig.url, {
            method: optionsConfig.method,
            ...d
        })
        if (res.code === 0) {
            setListData(res.data[optionsConfig.listName])
        }
    }

    useEffect(() => {
        if (data.designParams.optionsSourceType === 'custom') {
            getListData({})
        } else {
            setListData(optSource)
        }
    }, [props[data.designParams.optionsSourceName], data.fieldOption])

    return (
        <Form.Item
            className={`form-item form-item-radio ${data.fieldKey}`}
            rules={[{ required: data.fieldRequired, message: '请选择' }]}
            label={data.fieldDesc}
            name={data.formParams.field}
            extra={data.formParams.extra}
        >
            <Radio.Group
                disabled={data.formParams.disabled}
                onChange={(e) => {
                    props.change && props.change(data, e)
                }}
            >
                {listData.map((v) => {
                    return (
                        <Radio key={v.value} value={v.value}>
                            {v.label}
                        </Radio>
                    )
                })}
            </Radio.Group>
        </Form.Item>
    )
}

export default formRadio
