import React, { useMemo, useState, useEffect } from 'react'

import { Select, Form, Divider, Checkbox, FormInstance } from 'antd'

const { Option } = Select

const SelectView = (props: { data: any; form: FormInstance; [key: string]: any; request: any }) => {
    const { data, form, http } = props

    const [listData, setListData] = useState([])

    const optSource = useMemo(() => {
        const { optionsSourceType, optionsSourceName } = data.designParams

        if (optionsSourceType === 'static') {
            return data.fieldOption || []
        }
        const source = props[optionsSourceName] || []
        if (source && source.length) {
            return props[optionsSourceName]
        }
        console.error(`请传入${optionsSourceName}`)
        return []
    }, [props[data.designParams.optionsSourceName], data.fieldOption])

    const getListData = async (pm = {}) => {
        if (!props.request) {
            return
        }
        const { optionsConfig } = data
        const params = { [optionsConfig.field]: '', ...JSON.parse(optionsConfig.params || '{}'), ...pm }
        const d = optionsConfig.method === 'get' ? { params } : { data: params }
        const res = await props.request(data.optionsConfig.url, {
            method: optionsConfig.method,
            ...d
        })
        if (res.code === 0) {
            setListData(res.data[optionsConfig.listName])
        }
    }

    useEffect(() => {
        if (data.designParams.optionsSourceType === 'custom') {
            getListData({})
        } else {
            setListData(optSource)
        }
    }, [])

    return (
        <Form.Item
            className={`form-item form-item-select_multiple ${data.fieldKey}`}
            rules={[{ required: data.fieldRequired, message: '请选择' }]}
            label={data.fieldDesc}
            name={data.formParams.field}
            extra={data.formParams.extra}
        >
            <Select
                mode="multiple"
                placeholder={data.formParams.placeholder}
                showSearch={data.formParams.showSearch}
                allowClear
                dropdownRender={(menu) => {
                    if (data.formParams.isCheckAll) {
                        return (
                            <div>
                                {menu}
                                <Divider style={{ margin: '2px 0' }} />
                                <div style={{ padding: '4px 8px 8px 8px', cursor: 'pointer' }}>
                                    <Checkbox
                                        onChange={(e) => {
                                            props.form.setFieldsValue({
                                                [data.formParams.field]: e.target.checked
                                                    ? optSource.map((v) => {
                                                          return v.value
                                                      })
                                                    : []
                                            })
                                        }}
                                    >
                                        全选
                                    </Checkbox>
                                </div>
                            </div>
                        )
                    }
                    return menu
                }}
            >
                {listData.map((v, i) => {
                    return (
                        <Option key={i} value={v.value}>
                            {v.label}
                        </Option>
                    )
                })}
            </Select>
        </Form.Item>
    )
}

export default SelectView
