import React, { useMemo, useState, useEffect } from 'react'

import { Select, Form } from 'antd'

const { Option } = Select

const SelectView = (props) => {
    const { data } = props
    const [listData, setListData] = useState([])

    const optSource = useMemo(() => {
        const { optionsSourceType, optionsSourceName } = data.designParams
        if (optionsSourceType === 'static') {
            return data.fieldOption || []
        }
        const source = props[optionsSourceName] || []
        if (source && source.length) {
            return props[optionsSourceName]
        }
        console.error(`请传入${optionsSourceName}`)
        return []
    }, [props[data.designParams.optionsSourceName], data.fieldOption])

    const getListData = async (pm = {}) => {
        if (!props.request) {
            return
        }
        const { optionsConfig } = data
        const params = { [optionsConfig.field]: '', ...JSON.parse(optionsConfig.params || '{}'), ...pm }
        const d = optionsConfig.method === 'get' ? { params } : { data: params }
        const res = await props.request(data.optionsConfig.url, {
            method: optionsConfig.method,
            ...d
        })
        if (res.code === 0) {
            setListData(res.data[optionsConfig.listName])
        }
    }

    useEffect(() => {
        if (data.designParams.optionsSourceType === 'custom') {
            getListData({})
        } else {
            setListData(optSource)
        }
    }, [props[data.designParams.optionsSourceName], data.fieldOption])

    return (
        <Form.Item
            className={`form-item form-item-radio ${data.fieldKey}`}
            rules={[{ required: data.fieldRequired, message: '请选择' }]}
            label={data.fieldDesc}
            name={data.formParams.field}
            extra={data.formParams.extra}
        >
            <Select
                placeholder={data.formParams.placeholder}
                disabled={data.formParams.disabled}
                showSearch={data.formParams.showSearch}
                filterOption={(input, option) =>
                    option.children.toLowerCase().indexOf(input.toLowerCase()) >= 0
                }
                onSearch={(e) => {
                    console.log(e)
                }}
            >
                {listData.map((v) => {
                    return (
                        <Option key={v.value} value={v.value}>
                            {v.label}
                        </Option>
                    )
                })}
            </Select>
        </Form.Item>
    )
}

export default SelectView
