import React from 'react'

import { DesignItem, Icon, throttle } from '@/component/edit-engine/index'
import { Collapse, Input, Select } from 'antd'

import { getDragIndex, createUid } from '@/component/edit-engine/utils'

const { Panel } = Collapse

const EditTable = (props: any) => {
    const dragItem = (targetKey: string, goKey: string) => {
        const { targetIndex, goIndex } = getDragIndex(props.data.columns, targetKey, goKey, 'key')
        props.onChange((draft) => {
            const targetItem = draft.columns.splice(targetIndex, 1)
            draft.columns.splice(goIndex, 0, targetItem[0])
        })
    }

    const addItem = () => {
        props.onChange((draft) => {
            const val = `选项${draft.columns.length + 1}`
            const k = createUid()
            draft.columns.push({
                title: val,
                dataIndex: k,
                key: k,
                fieldKey: k
            })
        })
    }

    const deleteItem = (index) => {
        if (props.disablePage) {
            return
        }
        props.onChange((draft) => {
            draft.columns.splice(index, 1)
        })
    }

    const onChangeKeyArr = (name, value) => {
        props.onChange((draft) => {
            name.reduce((j, v) => {
                if (v === name[name.length - 1]) {
                    try {
                        j[v] = value
                    } catch (err) {
                        console.error('未定义的变量：先去对应组件的index.json 定义你要保存的变量')
                    }
                }
                return j[v]
            }, draft)
        })
    }

    return (
        <div className="form-edit_box edit-select-multiple">
            <div>
                <Collapse>
                    <Panel header="表格配置" key="3">
                        <div className="edit-box_item">
                            <DesignItem
                                className="edit-item_drag"
                                list={props.data.columns}
                                dragItem={throttle(dragItem, 500)}
                                keyName="key"
                                renderDom={(v, i) => {
                                    return (
                                        <div className="edit-item_edit">
                                            <div className="item-edit_box">
                                                <div className="item">
                                                    <div className="item-label">title</div>
                                                    <Input
                                                        defaultValue={v.title}
                                                        placeholder="请输入"
                                                        disabled={props.disablePage}
                                                        allowClear
                                                        onChange={(e) => {
                                                            onChangeKeyArr(
                                                                ['columns', i, 'title'],
                                                                e.target.value
                                                            )
                                                        }}
                                                    />
                                                </div>
                                                <div className="item">
                                                    <div className="item-label">key</div>
                                                    <Input
                                                        defaultValue={v.key}
                                                        placeholder="请输入"
                                                        disabled={props.disablePage}
                                                        allowClear
                                                        onChange={(e) => {
                                                            onChangeKeyArr(
                                                                ['columns', i, 'key'],
                                                                e.target.value
                                                            )
                                                            onChangeKeyArr(
                                                                ['columns', i, 'dataIndex'],
                                                                e.target.value
                                                            )
                                                        }}
                                                    />
                                                </div>
                                            </div>
                                            <div
                                                className="item-input_btn"
                                                onClick={() => {
                                                    deleteItem(i)
                                                }}
                                            >
                                                <Icon name="icon-_shanchu" />
                                            </div>
                                        </div>
                                    )
                                }}
                            ></DesignItem>
                            <div
                                className="edit-item_add"
                                onClick={() => {
                                    if (props.disablePage) {
                                        return
                                    }
                                    addItem()
                                }}
                            >
                                <Icon name="icon-tianjia" color="#fff" />
                                <div className="icon-add_text">添加选项</div>
                            </div>
                        </div>
                    </Panel>
                    <Panel header="列配置" key="1"></Panel>
                </Collapse>
            </div>
        </div>
    )
}

export default EditTable
