import React from 'react'

import { Input, Collapse, Select, Switch } from 'antd'

import DesignComm from '@/component/materiel-design-comm/shange'
import FormHeader from '@/component/materiel-design-comm/form-header'
import useDesignState from '@/component/edit-engine/Hooks/design-state'

import './index.less'

const { Panel } = Collapse
const { Option } = Select

const EditRadio = (props) => {
    const { onChangeKeyArr, deleteItem, addItem, dragItem } = useDesignState(props)
    const { data } = props

    const dataTypeOpt = () => {
        if (data.designParams.optionsSourceType === 'static') {
            return (
                <>
                    <Input.TextArea
                        defaultValue={JSON.stringify(data.fieldOption, undefined, 2)}
                    ></Input.TextArea>
                </>
            )
        } else if (data.designParams.optionsSourceType === 'dynamic') {
            return (
                <div>
                    <div style={{ margin: '10px 0' }}>数据名称</div>
                    <Input
                        onChange={(e) => {
                            onChangeKeyArr(['optionsSourceName'], e)
                        }}
                        defaultValue={data.designParams.optionsSourceName}
                        placeholder="根据这个名称传入对应数据"
                    ></Input>
                </div>
            )
        } else {
            return (
                <div className="custom-item_box">
                    <div style={{ margin: '10px 0' }}>远程options</div>
                    <div className="custom-item">
                        <div className="label">请求地址</div>
                        <Input defaultValue={data.optionsConfig.url} placeholder="/api"></Input>
                    </div>
                    <div className="custom-item">
                        <div className="label">请求方式</div>
                        <Select style={{ width: '100%' }} defaultValue={data.optionsConfig.method}>
                            <Option value="get">get</Option>
                            <Option value="post">post</Option>
                        </Select>
                    </div>
                    <div className="custom-item">
                        <div className="label">搜索字段名称</div>
                        <Input
                            placeholder="接口接收搜索的参数名称"
                            defaultValue={data.optionsConfig.field}
                        ></Input>
                    </div>
                    <div className="custom-item">
                        <div className="label">追加参数</div>
                        <Input.TextArea
                            placeholder="除搜索字段外别的参数"
                            defaultValue={data.optionsConfig.params}
                        ></Input.TextArea>
                    </div>
                    <div className="custom-item">
                        <div className="label">返回字段的名称</div>
                        <Input
                            placeholder="返回字段的名称"
                            defaultValue={data.optionsConfig.listName}
                        ></Input>
                    </div>
                </div>
            )
        }
    }
    return (
        <div className="form-edit_box edit-select-multiple">
            <div>
                <Collapse>
                    <Panel header="表单设置" key="3">
                        <FormHeader data={props.data} onChangeKeyArr={onChangeKeyArr} />
                        <div className="edit-box_item eidt-box_title">
                            <div className="box-item_label">模糊搜索:</div>
                            <Switch
                                defaultChecked={data.formParams.showSearch}
                                onChange={(e) => {
                                    onChangeKeyArr(['formParams', 'showSearch'], e)
                                }}
                            ></Switch>
                        </div>
                        <div className="edit-box_item eidt-box_title">
                            <div className="box-item_label">开启多选:</div>
                            <Switch
                                defaultChecked={data.formParams.multiple}
                                onChange={(e) => {
                                    onChangeKeyArr(['formParams', 'multiple'], e)
                                }}
                            ></Switch>
                        </div>
                        <div className="edit-box_item eidt-box_title">
                            <div className="box-item_label">引导文字:</div>
                            <Input
                                disabled={props.disablePage}
                                placeholder="请输入"
                                value={data.formParams.placeholder}
                                onChange={(e) => {
                                    onChangeKeyArr(['formParams', 'placeholder'], e.target.value)
                                }}
                            />
                        </div>
                        <div className="edit-box_item eidt-box_title">
                            <div className="box-item_label">显示字段:</div>
                            <Input
                                disabled={props.disablePage}
                                placeholder="请输入"
                                value={data.formParams.labelField}
                                onChange={(e) => {
                                    onChangeKeyArr(['formParams', 'labelField'], e.target.value)
                                }}
                            />
                        </div>
                        <div className="edit-box_item eidt-box_title">
                            <div className="box-item_label">提交字段:</div>
                            <Input
                                disabled={props.disablePage}
                                placeholder="请输入"
                                value={data.formParams.valueField}
                                onChange={(e) => {
                                    onChangeKeyArr(['formParams', 'valueField'], e.target.value)
                                }}
                            />
                        </div>
                    </Panel>
                    <Panel header="选项" key="1">
                        <div className="edit-box_item edit-box_group">
                            <div style={{ marginBottom: '10px' }}>
                                <div style={{ marginBottom: '10px' }}>数据类型</div>
                                <Select
                                    defaultValue={data.designParams.optionsSourceType}
                                    onChange={(e) => {
                                        onChangeKeyArr(['designParams', 'optionsSourceType'], e)
                                    }}
                                >
                                    <Option value="static">静态数据</Option>
                                    <Option value="dynamic">动态数据</Option>
                                    <Option value="custom">自定义数据</Option>
                                </Select>
                            </div>
                            {dataTypeOpt()}
                        </div>
                        {data.designParams.optionsSourceType === 'custom' ? (
                            <div className="edit-box_item edit-box_group">
                                <div style={{ marginBottom: '10px' }}>
                                    <div style={{ marginBottom: '10px' }}>option加载方式</div>
                                    <Select
                                        defaultValue={data.designParams.getOptionsSourceType}
                                        onChange={(e) => {
                                            onChangeKeyArr(['designParams', 'getOptionsSourceType'], e)
                                        }}
                                    >
                                        <Option value="allIn">一次加载</Option>
                                        <Option value="async">异步加载</Option>
                                    </Select>
                                </div>
                            </div>
                        ) : (
                            ''
                        )}
                    </Panel>
                    <Panel header="栅格布局" key="2">
                        <DesignComm data={data} onChangeKeyArr={onChangeKeyArr} />
                    </Panel>
                </Collapse>
            </div>
        </div>
    )
}

export default EditRadio
