import React, { useMemo,  useEffect } from 'react'

import { useImmer } from 'use-immer'
import { Select, Form,  FormInstance, TreeSelect } from 'antd'

const { Option } = Select
const TreeNode = TreeSelect.TreeNode

const TreeSelectView = (props: { data: any; form: FormInstance; [key: string]: any; request: any }) => {
    const { data, form, http } = props
    const [state, setState] = useImmer({ listData: [] })
    const optSource = useMemo(() => {
        const { optionsSourceType, optionsSourceName } = data.designParams
        if (optionsSourceType === 'static') {
            return data.fieldOption || []
        }
        const source = props[optionsSourceName] || []
        if (source && source.length) {
            return props[optionsSourceName]
        }
        console.error(`请传入${optionsSourceName}`)
        return []
    }, [props[data.designParams.optionsSourceName], data.fieldOption])

    const setListDataFunc = (list) => {
        setState((draft) => {
            draft.listData = list
        })
    }

    const getListData = async (pm = {}) => {
        if (!props.request) {
            console.error('请传入 request 方法')
            return [{ label: '北3京', value: '199000' }]
        }
        const { optionsConfig } = data
        const params = { [optionsConfig.field]: '', ...JSON.parse(optionsConfig.params || '{}'), ...pm }
        const d = optionsConfig.method === 'get' ? { params } : { data: params }
        const res = await props.request(data.optionsConfig.url, {
            method: optionsConfig.method,
            ...d
        })
        if (res.code === 0) {
            return res.data[optionsConfig.listName]
        }
        return []
    }

    const init = async () => {
        if (data.designParams.optionsSourceType === 'custom') {
            const data = await getListData({})
            setListDataFunc(data)
        } else {
            setState((draft) => {
                draft.listData = optSource
            })
        }
    }

    useEffect(() => {
        init()
    }, [props[data.designParams.optionsSourceName], data.fieldOption])

    // 渲染子组件
    const renderTreeNodes = (data) => {
        if (!(data && data.length)) {
            return
        }
        return data.map((item) => {
            const { children, ...reset } = item
            if (item.children) {
                return (
                    <TreeNode
                        key={item.key || item.value}
                        {...reset}
                        value={reset.value}
                        title={item.title}
                        dataRef={item}
                    >
                        {renderTreeNodes(item.children)}
                    </TreeNode>
                )
            }

            return <TreeNode key={item.key || item.value} value={item.value} dataRef={item} />
        })
    }

    const onLoadDat = async (node) => {
        const { value, pos, dataRef } = node.props
        const path = pos.split('-').map((v: string) => Number(v))
        path.shift()
        const data = await getListData({ [props.data.optionsConfig.field]: value })
        if (data.length) {
            setState((draft) => {
                let df: any = undefined
                path.forEach((v, i) => {
                    if (!df) {
                        df = draft.listData[v]
                    } else {
                        df = df['children'][v]
                    }
                    if (i === path.length - 1) {
                        df['children'] = data
                    }
                })
            })
        }
        return true
    }

    return (
        <Form.Item
            className={`form-item form-item-select_multiple ${data.fieldKey}`}
            rules={[{ required: data.fieldRequired, message: '请选择' }]}
            label={data.fieldDesc}
            name={data.formParams.field}
            extra={data.formParams.extra}
        >
            <TreeSelect
                treeNodeFilterProp={data.formParams.labelField}
                disabled={data.formParams.disabled}
                placeholder={data.formParams.placeholder}
                multiple={data.formParams.multiple}
                showSearch={data.formParams.showSearch}
                treeData={state.listData}
                allowClear
                labelInValue
                loadData={data.designParams.getOptionsSourceType === 'async' ? onLoadDat : undefined}
            >
                {renderTreeNodes(state.listData)}
            </TreeSelect>
        </Form.Item>
    )
}

export default TreeSelectView
