import React, { useRef, useState, useEffect } from 'react'

import { Select, message, Modal, Slider, Radio, Form, Input } from 'antd'
import Icon from '@/component/edit-engine/component/iconfont/icon'
import { useImmer } from 'use-immer'
import { EditEngine } from '@/component/edit-engine'

import Tools from './components/tools'

import meta from './meta.json'
import './index.less'

const { Option } = Select
const lay = meta.formLayout
    ? meta.formLayout
    : {
          labelCol: { span: undefined, offset: undefined, push: undefined, pull: undefined },
          wrapperCol: { span: undefined, offset: undefined, push: undefined, pull: undefined },
          layout: 'horizontal'
      }

const beforeFunc = function(e) {
    if (window.isCloseHint) {
        var confirmationMessage = '页面数据已被修改确定要离开吗？'
        ;(e || window.event).returnValue = confirmationMessage // 兼容 Gecko + IE
        return confirmationMessage // 兼容 Gecko + Webkit, Safari, Chrome
    }
}

const fetchFunc = (url, data) => {
    return fetch(`${window.location.origin}${url}`, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify(data)
    }).then((res) => res.json())
}

const Home = (props: any) => {
    const [data, setData] = useState({
        headerData: { formName: '' },
        formFieldList: []
    })
    const [formLayout, setFormLay] = useImmer(lay)
    const [isModalVisible, setisModalVisible] = useState(false)
    const [addModal, setAddModal] = useState(false)
    const [importModal, setImportModal] = useState(false)
    const [metaList, setMetaList] = useState([])
    const [formName, setFormName] = useState('')
    const [ipJson, setIpJson] = useState('')
    const [ipName, setIpName] = useState('')

    const [form] = Form.useForm()
    const editRef = useRef()

    const engineChange = (e) => {
        window.isCloseHint = e
    }

    const localSave = async (name) => {
        /**TODO: 保存的时候可能设计器生成的json, 和后端要求的格式不太一样。建议直接通过 editRef.current.getSuccessData()获取到后，在这里修改成符合要求的,提交给后端接口。
         * TODO: 如果非改不可，需要把所有组件有用到这个的参数都改。注意开发者模式生成的预览代码格式需要变一下。path: cli/generate-form-tsx.js
         * TODO: "formFieldExtra":{ // colProps、designParams、formParams、optionsConfig  这几个参数design是object。且是独立的。后端要求需要JSON.stringify().并且包裹在formFieldExtra
                      "colProps": "",
                      "optionsConfig": "",
                      "formParams": "",
                      "designParams": ""
                  }
         */
        const { formFieldList = [] } = editRef.current.getSuccessData()
        if (!formFieldList.length) {
            message.warn('保存了空气！')
            return
        }
        const hide = message.loading('保存中...')
        const res = await fetchFunc('/modify-mate', {
            list: formFieldList,
            formLayout,
            formName: name || localStorage.getItem('formName')
        })
        if (res.code === 0) {
            localStorage.setItem('formName', res.data.name)
        }
        window.isCloseHint = false
        hide()
    }

    const build = async () => {
        const { formFieldList = [] } = editRef.current.getSuccessData()
        if (!formFieldList.length) {
            message.warn('打包了空气！')
            return
        }
        const hide = message.loading('保存中...')
        await fetchFunc('/build-page', {
            list: formFieldList,
            formLayout,
            formName: localStorage.getItem('formName')
        })
        getMetaListFunc()
        hide()
    }

    const addForm = async () => {
        try {
            const values = await form.validateFields()
            const res = await fetchFunc('/add-meta', values)
            if (res.code === 0) {
                setMetaList(res.data.metaList)
                setAddModal(false)
            }
        } catch (errorInfo) {
            console.log(errorInfo)
        }
    }

    const getMetaListFunc = async () => {
        const res = await fetchFunc('/get-meta-list', {})
        const { metaList = [] } = res.data
        const list = metaList.filter((v) => v !== '普通模式专用')
        setMetaList(list)
        getMetaDetilas(list[0])
    }

    const getMetaDetilas = async (formName) => {
        if (!formName) {
            setFormLay({
                labelCol: {},
                wrapperCol: {},
                layout: 'horizontal'
            })
            setData({
                headerData: { formName: '' },
                formFieldList: []
            })
            setFormName('')
            localStorage.setItem('formName', '')
            return
        }
        /**
         * TODO: 获取的时候如果有数据需要转换，可以在这里转好在传给 EditEngine 组件的data
         */
        const metaData = await fetchFunc('/get-meta-details', { formName })
        const { details } = metaData.data
        setFormLay(details.formLayout)
        setData({
            headerData: { formName: details.name },
            formFieldList: details.list
        })
        setFormName(formName)
        localStorage.setItem('formName', formName)
        window.isCloseHint = false
    }

    const typeChange = async (e) => {
        if (e === '1') {
            setData({
                headerData: { formName: '' },
                formFieldList: []
            })
            localStorage.setItem('formName', '普通模式专用')
            getMetaDetilas('普通模式专用')
            return
        }
        getMetaListFunc()
    }

    const formChange = async (e) => {
        getMetaDetilas(e)
        localStorage.setItem('formName', e)
    }

    const init = async () => {
        const type = localStorage.getItem('pageType')
        if (type === '2') {
            getMetaListFunc()
        } else {
            getMetaDetilas('普通模式专用')
        }
    }

    const ipFunc = async () => {
        try {
            const data = JSON.parse(ipJson)
            const res = await fetchFunc('/import-json', { data, name: ipName })
            getMetaDetilas(res.data.name)
            localStorage.setItem('formName', res.data.name)
            setImportModal(false)
        } catch (err) {
            setImportModal(false)
            alert('格式错误')
        }
    }

    const importJson = () => {
        setImportModal(true)
    }

    useEffect(() => {
        window.addEventListener('beforeunload', beforeFunc)
        init()
        return () => {
            window.removeEventListener('beforeunload', beforeFunc)
        }
    }, [])

    return (
        <div className="home">
            <EditEngine data={data} formLayout={formLayout} ref={editRef} change={engineChange} />
            <Tools
                typeChange={typeChange}
                formChange={formChange}
                setisModalVisible={setisModalVisible}
                metaList={metaList}
                addForm={() => {
                    setAddModal(true)
                }}
                preview={() => {
                    window.open(window.location.origin + '/preview')
                }}
                importJson={() => {
                    importJson()
                }}
                formName={formName}
                localSave={localSave}
                build={build}
            />

            <Modal
                style={{ top: 20 }}
                title="from 布局"
                className="form-lay"
                visible={isModalVisible}
                onOk={() => {}}
                footer={[]}
                onCancel={() => {
                    setisModalVisible(false)
                }}
            >
                <div style={{ margin: '15px 0' }}>
                    <div className="modal-label">label 位置</div>
                    <div className="modal-content">
                        <Radio.Group
                            value={formLayout.layout}
                            onChange={(e) => {
                                setFormLay((draft) => {
                                    draft.layout = e.target.value
                                })
                            }}
                        >
                            <Radio.Button value="vertical">顶部</Radio.Button>
                            <Radio.Button value="horizontal">平行</Radio.Button>
                        </Radio.Group>
                    </div>
                    <div className="modal-label">label:</div>
                    <div className="modal-content">
                        <div className="box-item_children">
                            <div className="item-children_i">
                                <div className="children_label">
                                    栅格格数<span className="tip">共24列</span>
                                </div>
                                <Slider
                                    max={24}
                                    defaultValue={formLayout.labelCol.span}
                                    onChange={(e) => {
                                        setFormLay((draft) => {
                                            draft.labelCol.span = e
                                        })
                                    }}
                                />
                            </div>
                            <div className="item-children_i">
                                <div className="children_label">
                                    左侧间隔格数
                                    <span className="tip">0-24之间</span>
                                </div>
                                <Slider
                                    max={24}
                                    defaultValue={formLayout.labelCol.offset}
                                    onChange={(e) => {
                                        setFormLay((draft) => {
                                            draft.labelCol.offset = e
                                        })
                                    }}
                                />
                            </div>
                            <div className="item-children_i">
                                <div className="children_label">
                                    右移格数<span className="tip">0-24之间</span>
                                </div>
                                <Slider
                                    max={24}
                                    defaultValue={formLayout.labelCol.push}
                                    onChange={(e) => {
                                        setFormLay((draft) => {
                                            draft.labelCol.push = e
                                        })
                                    }}
                                />
                            </div>
                            <div className="item-children_i">
                                <div className="children_label">
                                    左移格数<span className="tip">0-24之间</span>
                                </div>
                                <Slider
                                    max={24}
                                    defaultValue={formLayout.labelCol.pull}
                                    onChange={(e) => {
                                        setFormLay((draft) => {
                                            draft.labelCol.pull = e
                                        })
                                    }}
                                />
                            </div>
                        </div>
                    </div>
                    <div className="modal-label">wrapperCol</div>
                    <div className="modal-content">
                        <div className="box-item_children">
                            <div className="item-children_i">
                                <div className="children_label">
                                    栅格格数<span className="tip">共24列</span>
                                </div>
                                <Slider
                                    max={24}
                                    defaultValue={formLayout.wrapperCol.span}
                                    onChange={(e) => {
                                        setFormLay((draft) => {
                                            draft.wrapperCol.span = e
                                        })
                                    }}
                                />
                            </div>
                            <div className="item-children_i">
                                <div className="children_label">
                                    左侧间隔格数
                                    <span className="tip">0-24之间</span>
                                </div>
                                <Slider
                                    max={24}
                                    defaultValue={formLayout.wrapperCol.offset}
                                    onChange={(e) => {
                                        setFormLay((draft) => {
                                            draft.wrapperCol.offset = e
                                        })
                                    }}
                                />
                            </div>
                            <div className="item-children_i">
                                <div className="children_label">
                                    右移格数<span className="tip">0-24之间</span>
                                </div>
                                <Slider
                                    max={24}
                                    defaultValue={formLayout.wrapperCol.push}
                                    onChange={(e) => {
                                        setFormLay((draft) => {
                                            draft.wrapperCol.push = e
                                        })
                                    }}
                                />
                            </div>
                            <div className="item-children_i">
                                <div className="children_label">
                                    左移格数<span className="tip">0-24之间</span>
                                </div>
                                <Slider
                                    max={24}
                                    defaultValue={formLayout.wrapperCol.pull}
                                    onChange={(e) => {
                                        setFormLay((draft) => {
                                            draft.wrapperCol.pull = e
                                        })
                                    }}
                                />
                            </div>
                        </div>
                    </div>
                </div>
            </Modal>
            <Modal
                title="导入表单"
                visible={importModal}
                style={{top: 20}}
                onOk={() => {
                    ipFunc()
                }}
                onCancel={() => {
                    setImportModal(false)
                }}
            >
                <Form>
                    <Form.Item label="表单名称">
                        <Input
                            value={ipName}
                            onChange={(e) => {
                                setIpName(e.target.value)
                            }}
                        />
                    </Form.Item>
                    <Form.Item label="json数组">
                        <Input.TextArea
                            value={ipJson}
                            rows={20}
                            onChange={(e) => {
                                setIpJson(e.target.value)
                            }}
                        ></Input.TextArea>
                    </Form.Item>
                </Form>
            </Modal>
            <Modal
                style={{ top: 20, height: '200px' }}
                cancelText="取消"
                okText="保存"
                title="新增表单"
                visible={addModal}
                onCancel={() => {
                    setAddModal(false)
                }}
                onOk={addForm}
            >
                <div>
                    <Form form={form}>
                        <Form.Item
                            label="表单名称"
                            name="name"
                            rules={[{ required: true, message: '请输入表单名称' }]}
                        >
                            <Input placeholder="请输入表单名称"></Input>
                        </Form.Item>
                        <Form.Item
                            label="表单说明"
                            name="doc"
                            rules={[{ required: true, message: '请输入表单说明' }]}
                        >
                            <Input placeholder="请输入表单说明"></Input>
                        </Form.Item>
                    </Form>
                </div>
            </Modal>
        </div>
    )
}

export default Home
